@@ -2,6 +2,7 @@ from __future__ import absolute_import, print_function, unicode_literals
 
 import json
 import os
+import re
 import sys
 from glob import glob
 
@@ -47,7 +48,7 @@ def get_topology_definition(topology_name=None):
                 "explicitly specify the topology by name using the -n or "
                 "--name flags.".format(specs_dir=topology_path))
         topology_file = topology_files[0]
-        topology_name = topology_file.rstrip(".clj").lstrip(topology_path)
+        topology_name = re.sub(r'(^{}|\.clj$)'.format(topology_path), '', topology_file)
     else:
         topology_file = "{}.clj".format(os.path.join(topology_path, topology_name))
         if not os.path.exists(topology_file):
@@ -67,7 +68,7 @@ def get_env_config(env_name=None):
     """
     config = get_config()
     if env_name is None and len(config["envs"]) == 1:
-        env_name = config["envs"].keys()[0]
+        env_name = list(config["envs"].keys())[0]
     elif env_name is None and len(config["envs"]) > 1:
         die("Found more than one environment in config.json. "
             "When more than one environment exists, you must "
@@ -85,7 +86,7 @@ def get_nimbus_for_env_config(env_config):
     config file.
     """
     if not env_config["nimbus"]:
-        die("No Nimbus server configured for in config.json.")
+        die("No Nimbus server configured in config.json.")
 
     if ":" in env_config["nimbus"]:
         host, port = env_config["nimbus"].split(":", 1)
