@@ -12,6 +12,11 @@ from streamparse.bolt import BatchingBolt
 
 class DummyBatchingBolt(BatchingBolt):
 
+    secs_between_batches = 1
+    auto_ack = False
+    auto_anchor = False
+    auto_fail = False
+
     def group_key(self, tup):
         return tup.values[0]
 
